from web.forms import SubjectAddForm
from web.forms import SubjectEditForm
from web.models import StudySubject
from web.models.constants import SEX_CHOICES_MALE, SUBJECT_TYPE_CHOICES_CONTROL, COUNTRY_AFGHANISTAN_ID
from web.tests import LoggedInWithWorkerTestCase


class SubjectEditFormTests(LoggedInWithWorkerTestCase):
    def setUp(self):
        super(SubjectEditFormTests, self).setUp()

        location = self.worker.locations.all()[0]
        self.sample_data = {'first_name': 'name',
                            'last_name': 'name',
                            'sex': SEX_CHOICES_MALE,
                            'type': SUBJECT_TYPE_CHOICES_CONTROL,
                            'default_location': location.id,
                            'country': COUNTRY_AFGHANISTAN_ID,
                            'screening_number': '123',
                            'nd_number': 'ND0123'
                            }

    def tearDown(self):
        StudySubject.objects.all().delete()

    def test_validation(self):
        add_form = SubjectAddForm(data=self.sample_data, user=self.user)
        subject = add_form.save()
        self.sample_data['id'] = subject.id

        edit_form = SubjectEditForm(self.sample_data)
        save_status = edit_form.is_valid()
        self.assertTrue(save_status)

    def test_invalid_nd_number_edit(self):
        add_form = SubjectAddForm(data=self.sample_data, user=self.user)
        add_form.save()

        self.sample_data['nd_number'] = "ND0124"
        self.sample_data['screening_number'] = "124"
        add_form = SubjectAddForm(data=self.sample_data, user=self.user)
        subject = add_form.save()

        self.sample_data['id'] = subject.id
        self.sample_data['nd_number'] = "ND0123"
        edit_form = SubjectEditForm(self.sample_data)

        save_status = edit_form.is_valid()
        self.assertFalse(save_status)

    def test_invalid_mpower_id_edit(self):
        self.sample_data['screening_number'] = "001"
        add_form = SubjectAddForm(data=self.sample_data, user=self.user)
        subject = add_form.save()

        self.sample_data['mpower_id'] = "mpower_002"
        self.sample_data['nd_number'] = "ND0002"
        self.sample_data['screening_number'] = "002"
        add_form = SubjectAddForm(data=self.sample_data, user=self.user)
        add_form.save()

        self.sample_data['id'] = subject.id
        self.sample_data['mpower_id'] = "mpower_002"
        self.sample_data['nd_number'] = "ND0001"
        self.sample_data['screening_number'] = "001"
        edit_form = SubjectEditForm(self.sample_data)

        save_status = edit_form.is_valid()
        self.assertFalse(save_status)
