import datetime

from django.test import TestCase

from web.models.constants import SUBJECT_TYPE_CHOICES_PATIENT
from web.models import Visit
from web.tests.functions import create_subject, create_visit
from web.views.notifications import get_today_midnight_date


class VisitModelTests(TestCase):
    def test_mark_as_finished(self):
        subject = create_subject()
        visit = create_visit(subject)

        visit.mark_as_finished()

        visit_count = Visit.objects.filter(subject=subject).count()
        self.assertEquals(2, visit_count)

    def test_mark_as_finished_2(self):
        subject = create_subject()
        visit = create_visit(subject)
        subject.dead = True
        subject.save()

        visit.mark_as_finished()

        visit_count = Visit.objects.filter(subject=subject).count()
        self.assertEquals(1, visit_count)

    def test_mark_as_finished_3(self):
        subject = create_subject()
        visit = create_visit(subject)
        subject.resigned = True
        subject.save()

        visit.mark_as_finished()

        visit_count = Visit.objects.filter(subject=subject).count()
        self.assertEquals(1, visit_count)

    def test_mark_as_finished_for_follow_up_visit(self):
        subject = create_subject()
        subject.type = SUBJECT_TYPE_CHOICES_PATIENT
        subject.save()
        visit = create_visit(subject)

        visit.mark_as_finished()

        follow_up_visit = Visit.objects.filter(subject=subject).filter(visit_number=2)[0]
        follow_up_visit.datetime_begin = visit.datetime_begin + datetime.timedelta(days=133)
        follow_up_visit.datetime_end = visit.datetime_begin + datetime.timedelta(days=170)
        follow_up_visit.save()

        follow_up_visit.mark_as_finished()

        visit_count = Visit.objects.filter(subject=subject).count()
        self.assertEquals(3, visit_count)

        new_follow_up = Visit.objects.filter(subject=subject).filter(visit_number=3)[0]

        # check if follow up date is based on the first visit date
        self.assertTrue(visit.datetime_begin + datetime.timedelta(days=365 * 2 - 1) < new_follow_up.datetime_begin)

    def test_visit_to_string(self):
        visit = create_visit(create_subject())
        self.assertIsNotNone(str(visit))

    def test_visit_to_unicode(self):
        visit = create_visit(create_subject())
        self.assertIsNotNone(unicode(visit))
